@@ -22,8 +22,8 @@ class BooleanFieldDefinitionTestMixin(FieldDefinitionTestMixin):
 
     @skipIf(
         connection.settings_dict['ENGINE'] == 'django.db.backends.sqlite3' and
-        south.__version__ == '0.8.1',
-        "South 0.8.1 doesn't escape added column default value correctly on SQLite3."
+        south.__version__ in ('0.8.1', '0.8.2', '0.8.3', '0.8.4'),
+        "This version of South doesn't escape added column default value correctly on SQLite3."
     )
     def test_create_with_default(self):
         super(BooleanFieldDefinitionTestMixin, self).test_create_with_default()
@@ -33,7 +33,7 @@ class BooleanFieldDefinitionTest(BooleanFieldDefinitionTestMixin,
                                  BaseModelDefinitionTestCase):
     field_definition_cls = BooleanFieldDefinition
     field_definition_init_kwargs = {'default': True}
-    field_values = (True, False)
+    field_values = (False, True)
 
 
 class NullBooleanFieldDefinitionTest(BooleanFieldDefinitionTestMixin,
