@@ -3,31 +3,18 @@ from __future__ import unicode_literals
 from django.db.models import Q, signals
 from django.db.models.fields.related import RelatedField
 from django.dispatch.dispatcher import receiver
+from south.db import dbs
 
 from ....db.models import MutableModel
-from ....management import allow_syncdbs, perform_ddl
+from ....management import perform_ddl
 from ....models import ModelDefinition
 from ....signals import mutable_class_prepared
+from ....utils import allow_migrate, clear_opts_related_cache
 
 from ..models import (ForeignKeyDefinition, ManyToManyFieldDefinition,
     OneToOneFieldDefinition)
 
 
-_opts_related_cache_attrs = ('_related_objects_cache', '_related_objects_proxy_cache',
-                             '_related_many_to_many_cache', '_name_map')
-
-def clear_opts_related_cache(model_class):
-    """
-    Clear the specified model opts related cache
-    """
-    opts = model_class._meta
-    for attr in _opts_related_cache_attrs:
-        try:
-            delattr(opts, attr)
-        except AttributeError:
-            pass
-
-
 @receiver(mutable_class_prepared)
 def mutable_model_prepared(signal, sender, definition, existing_model_class,
                            **kwargs):
@@ -73,7 +60,7 @@ def many_to_many_field_definition_pre_delete(sender, instance, **kwargs):
     field = model_class._meta.get_field(str(instance.name))
     intermediary_table_name = field.rel.through._meta.db_table
     instance._state._m2m_deletion = (
-        allow_syncdbs(model_class),
+        allow_migrate(model_class),
         intermediary_table_name
     )
 
@@ -81,7 +68,8 @@ def many_to_many_field_definition_pre_delete(sender, instance, **kwargs):
 @receiver(signals.post_delete, sender=ManyToManyFieldDefinition,
           dispatch_uid='mutant.contrib.related.management.many_to_many_field_definition_post_delete')
 def many_to_many_field_definition_post_delete(sender, instance, **kwargs):
-    syncdbs, intermediary_table_name = instance._state._m2m_deletion
-    for db in syncdbs:
+    aliases, intermediary_table_name = instance._state._m2m_deletion
+    for alias in aliases:
+        db = dbs[alias]
         db.delete_table(intermediary_table_name)
     del instance._state._m2m_deletion
