@@ -5,13 +5,13 @@ from django.db import models
 from django.utils.translation import ugettext_lazy as _
 
 from .. import logger
-from ..utils import remove_from_app_cache
+from ..state import handler as state_handler
 
 
 class MutableModel(models.Model):
-    """
-    Abstract class used to identify models that we're created by a definition
-    """
+    """Abstract class used to identify models that we're created by a
+    definition."""
+
     class Meta:
         abstract = True
 
@@ -21,24 +21,34 @@ class MutableModel(models.Model):
         return definition_cls.objects.get(pk=definition_pk)
 
     @classmethod
-    def is_obsolete(cls):
-        return cls._is_obsolete
+    def checksum(cls):
+        return cls._checksum
 
     @classmethod
-    def mark_as_obsolete(cls):
-        remove_from_app_cache(cls)
+    def is_obsolete(cls):
+        return (
+            cls._is_obsolete or
+            cls._checksum != state_handler.get_checksum(cls._definition[1])
+        )
+
+    @classmethod
+    def mark_as_obsolete(cls, origin=None):
         cls._is_obsolete = True
         logger.debug(
             "Marking model %s and it dependencies (%s) as obsolete.",
             cls, cls._dependencies
         )
+        if origin is None:
+            origin = cls._definition
         for definition_cls, definition_pk in cls._dependencies:
+            if (definition_cls, definition_pk) == origin:
+                continue
             try:
                 definition = definition_cls.objects.get(pk=definition_pk)
             except definition_cls.DoesNotExist:
                 pass
             else:
-                definition.model_class().mark_as_obsolete()
+                definition.model_class().mark_as_obsolete(origin)
 
     def clean(self):
         if self.is_obsolete():
