@@ -14,20 +14,15 @@ from south.db import dbs
 from .. import logger
 from ..models import (ModelDefinition, BaseDefinition, FieldDefinition,
     UniqueTogetherDefinition)
-from ..utils import popattr
-
-
-def allow_syncdbs(model):
-    for alias, db in dbs.iteritems():
-        if router.allow_syncdb(alias, model):
-            yield db
+from ..utils import allow_migrate, popattr
 
 
 def perform_ddl(model, action, *args, **kwargs):
     if model._meta.managed:
         return
 
-    for db in allow_syncdbs(model):
+    for alias in allow_migrate(model):
+        db = dbs[alias]
         if db.deferred_sql:
             for statement in db.deferred_sql:
                 logger.warn("Clearing non-executed deferred SQL statement "
@@ -81,6 +76,11 @@ def model_definition_post_save(sender, instance, created, **kwargs):
         else:
             for column, field in extra_fields:
                 if field.primary_key:
+                    assert isinstance(primary_key, models.AutoField)
+                    primary_key = field
+                elif (field.rel and field.rel.parent_link and
+                      isinstance(primary_key, models.AutoField)):
+                    field.primary_key = True
                     primary_key = field
                 else:
                     fields.append((column, field))
@@ -139,9 +139,26 @@ def base_definition_post_save(sender, instance, created, raw, **kwargs):
                 delattr(instance._state, '_add_columns')
             finally:
                 if add_columns:
+                    auto_pk = isinstance(opts.pk, models.AutoField)
                     for field in declared_fields:
-                        perform_ddl(model_class, 'add_column', table_name,
-                                    field.name, field)
+                        if auto_pk and field.rel and field.rel.parent_link:
+                            auto_pk = False
+                            field.primary_key = True
+                            auto_pk_column = opts.pk.get_attname_column()[1]
+                            perform_ddl(
+                                model_class, 'alter_column', table_name,
+                                auto_pk_column, field
+                            )
+                            column = field.get_attname_column()[1]
+                            perform_ddl(
+                                model_class, 'rename_column', table_name,
+                                auto_pk_column, column
+                            )
+                        else:
+                            perform_ddl(
+                                model_class, 'add_column', table_name,
+                                field.name, field
+                            )
         else:
             for field in declared_fields:
                 try:
@@ -232,9 +249,7 @@ def field_definition_post_save(sender, instance, created, raw, **kwargs):
     This signal is connected by all FieldDefinition subclasses
     see comment in FieldDefinitionBase for more details
     """
-    # If the field definition is raw we must re-create the model class
-    # since ModelDefinitionAttribute.save won't be called
-    model_class = instance.model_def.model_class(force_create=raw)
+    model_class = instance.model_def.model_class()
     table_name = model_class._meta.db_table
     field = instance._south_ready_field_instance()
     if created:
