@@ -1,7 +1,9 @@
 from __future__ import unicode_literals
 
+from hashlib import md5
 from inspect import isclass
 from itertools import chain
+import pickle
 
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ValidationError, ImproperlyConfigured
@@ -9,7 +11,6 @@ from django.db import models
 from django.db.models.fields import FieldDoesNotExist
 from django.db.models.signals import class_prepared
 from django.utils.translation import ugettext_lazy as _
-from orderable.models import OrderableModel
 from picklefield.fields import PickledObjectField
 
 try:
@@ -22,7 +23,10 @@ from ...db.deletion import CASCADE_MARK_ORIGIN
 from ...db.fields import LazilyTranslatedField, PythonIdentifierField
 from ...db.models import MutableModel
 from ...signals import mutable_class_prepared
-from ...utils import get_db_table, model_name
+from ...state import handler as state_handler
+from ...utils import get_db_table, model_name, remove_from_app_cache
+
+from ..ordered import OrderedModel
 
 from .managers import ModelDefinitionManager
 
@@ -38,89 +42,107 @@ def _model_class_from_pk(definition_cls, definition_pk):
         pass
 
 
-class _ModelClassProxy(object):
-    def __init__(self, model_class):
-        self.__dict__['model_class'] = model_class
+class MutableModelProxy(object):
+    __slots__ = ['model', 'refreshing', '__weakref__']
 
-    @property
-    def __model_class_is_obsolete(self):
-        return self.model_class.is_obsolete()
+    proxied_methods = [
+        '__setattr__', '__delattr__', '__hash__', '__str__'
+    ]
 
-    @staticmethod
-    def __get_underlying_model_class(value):
-        if isinstance(value, _ModelClassProxy):
-            return value.model_class
-        elif isclass(value) and issubclass(value, MutableModel):
-            return value
+    @classmethod
+    def method_factory(cls, name):
+        def method(self, *args, **kwargs):
+            model = self.__get__()
+            return getattr(model.__class__, name)(model, *args, **kwargs)
+        method.__name__ = str(name)
+        return method
 
-    def __get_model_class(self):
-        if self.__model_class_is_obsolete:
-            return self.__get__(None, None)
-        else:
-            return self.model_class
+    @classmethod
+    def factory(cls, base):
+        attrs = dict(
+            (name, cls.method_factory(name)) for name in cls.proxied_methods
+        )
+        name = str("%s(%s)" % (cls.__name__, base.__name__))
+        return type(name, (cls,), attrs)
 
-    def __get__(self, instance, owner):
-        if self.__model_class_is_obsolete:
+    def __new__(cls, model, *args, **kwargs):
+        base = model.__class__
+        try:
+            cache = cls._proxy_class_cache
+        except AttributeError:
+            cls._proxy_class_cache = cache = {}
+        try:
+            proxy_class = cache[base]
+        except KeyError:
+            cache[base] = proxy_class = cls.factory(base)
+        proxy = super(MutableModelProxy, cls).__new__(proxy_class)
+        proxy_class.__init__(proxy, model, *args, **kwargs)
+        return proxy
+
+    def __init__(self, model):
+        assert issubclass(model, MutableModel)
+        supset = super(MutableModelProxy, self).__setattr__
+        supset('model', model)
+        supset('refreshing', False)
+
+    def __get__(self, instance=None, owner=None):
+        model = self.model
+        if not self.refreshing and model.is_obsolete():
+            supset = super(MutableModelProxy, self).__setattr__
             try:
-                definition = self.model_class.definition()
+                supset('refreshing', True)
+                try:
+                    definition = model.definition()
+                finally:
+                    supset('refreshing', False)
             except ModelDefinition.DoesNotExist:
                 raise AttributeError('This model definition has been deleted')
             else:
-                self.__dict__['model_class'] = definition.model_class()
-        return self.model_class
+                proxy = definition.model_class()
+                assert isinstance(proxy, MutableModelProxy)
+                model = proxy.model
+                supset('model', model)
+        return model
 
-    def __set__(self, instance, value):
-        model_class = self.__get_underlying_model_class(value)
-        if model_class is not None:
-            self.model_class = model_class
-        else:
-            raise AttributeError('Invalid value')
+    def __getattribute__(self, name):
+        if name in ('model', 'refreshing', '__get__', '__reduce_ex__'):
+            return super(MutableModelProxy, self).__getattribute__(name)
+        model = super(MutableModelProxy, self).__getattribute__('__get__')()
+        return getattr(model, name)
 
     def __call__(self, *args, **kwargs):
-        model_class = self.__get_model_class()
-        return model_class(*args, **kwargs)
-
-    def __getattr__(self, name):
-        model_class = self.__get_model_class()
-        return getattr(model_class, name)
-
-    def __setattr__(self, name, value):
-        model_class = self.__get_model_class()
-        return setattr(model_class, name, value)
-
-    def __delattr__(self, name):
-        model_class = self.__get_model_class()
-        return delattr(model_class, name)
-
-    def __instancecheck__(self, instance):
-        model_class = self.__get_model_class()
-        return isinstance(instance, model_class)
+        model = self.__get__()
+        return model(*args, **kwargs)
 
     def __eq__(self, other):
-        other_model_class = self.__get_underlying_model_class(other)
-        if type(self.model_class) == type(other_model_class):
-            return self.model_class == other_model_class
-        else:
-            return NotImplemented
+        model = self.__get__()
+        if isinstance(other, MutableModelProxy):
+            other = other.model
+        if type(model) == type(other):
+            return model == other
+        return NotImplemented
 
-    def __reduce__(self):
-        model_class = self.__get_model_class()
-        return (_model_class_from_pk, model_class._definition)
+    def __instancecheck__(self, instance):
+        model = self.__get__()
+        return isinstance(instance, model)
 
-    def __str__(self):
-        model_class = self.__get_model_class()
-        return str(model_class)
+    def __reduce_ex__(self, protocol):
+        model = self.__get__()
+        return (_model_class_from_pk, model._definition)
 
 
 class ModelDefinition(ContentType):
     object_name = PythonIdentifierField(_('object name'))
-    db_table = models.CharField(_('database table'), max_length=63,
-                                blank=True, null=True)
+    db_table = models.CharField(
+        _('database table'), max_length=63, blank=True, null=True
+    )
     managed = models.BooleanField(_('managed'), default=False)
-    verbose_name = LazilyTranslatedField(_('verbose name'),
-                                         blank=True, null=True)
-    verbose_name_plural = LazilyTranslatedField(_('verbose name plural'),
-                                                blank=True, null=True)
+    verbose_name = LazilyTranslatedField(
+        _('verbose name'), blank=True, null=True
+    )
+    verbose_name_plural = LazilyTranslatedField(
+        _('verbose name plural'), blank=True, null=True
+    )
 
     objects = ModelDefinitionManager()
 
@@ -130,11 +152,11 @@ class ModelDefinition(ContentType):
         verbose_name_plural = _('model definitions')
 
     def __unicode__(self):
-        return '.'.join((self.app_label, self.object_name))
+        return "%s.%s" % (self.app_label, self.object_name)
 
     def natural_key(self):
         return (self.app_label, self.model)
-    natural_key.dependencies = ('contenttypes.contenttype',)
+    natural_key.dependencies = ['contenttypes.contenttype']
 
     def __init__(self, *args, **kwargs):
         # Attach unsaved related objects
@@ -165,70 +187,109 @@ class ModelDefinition(ContentType):
             self._model_class = super(ModelDefinition, self).model_class()
 
     def get_model_bases(self):
-        return tuple(bd.get_model_class() for bd in self.basedefinitions.all())
+        """Build a tuple of bases for the constructed definition"""
+        bases = []
+        has_mutable_base = False
+        for base_def in self.basedefinitions.all():
+            base = base_def.construct()
+            if issubclass(base, MutableModel):
+                has_mutable_base = True
+                base._dependencies.add((self.__class__, self.pk))
+            bases.append(base)
+        if not has_mutable_base:
+            bases.append(MutableModel)
+        return tuple(bases)
 
     def get_model_opts(self):
-        attrs = {'app_label': self.app_label, 'managed': self.managed}
+        opts = {'app_label': self.app_label, 'managed': self.managed}
         # Database table
         db_table = self.db_table
         if db_table is None:
             db_table = get_db_table(*self.natural_key())
-        attrs['db_table'] = db_table
+        opts['db_table'] = db_table
         # Verbose names
         if self.verbose_name is not None:
-            attrs['verbose_name'] = self.verbose_name
+            opts['verbose_name'] = self.verbose_name
         if self.verbose_name_plural is not None:
-            attrs['verbose_name_plural'] = self.verbose_name_plural
-        # Unique togethers
-        unique_together = tuple(tuple(utd.field_defs.names())
-                                    for utd in self.uniquetogetherdefinitions.all())
+            opts['verbose_name_plural'] = self.verbose_name_plural
+        # Unique together
+        unique_together = tuple(
+            ut_def.construct()
+            for ut_def in self.uniquetogetherdefinitions.all()
+        )
         if unique_together:
-            attrs['unique_together'] = unique_together
+            opts['unique_together'] = unique_together
         # Ordering
-        ordering = tuple(ordf.get_defined_ordering()
-                            for ordf in self.orderingfielddefinitions.all())
+        ordering = tuple(
+            ord_field_def.construct()
+            for ord_field_def in self.orderingfielddefinitions.all()
+        )
         if ordering:
             # Make sure not to add ordering if it's empty since it would
-            # prevent the model from inheriting it's possible base ordering.
+            # prevent the model from inheriting its possible base ordering.
             # Kinda related to django #17429
-            attrs['ordering'] = ordering
-        return type(str('Meta'), (), attrs)
+            opts['ordering'] = ordering
+        return opts
 
-    def get_model_attrs(self, existing_model_class=None):
+    def get_model_attrs(self):
         attrs = {
-            'Meta': self.get_model_opts(),
             '__module__': str("mutant.apps.%s.models" % self.app_label),
             '_definition': (self.__class__, self.pk),
-            '_is_obsolete': False,
             '_dependencies': set(),
+            '_is_obsolete': False
         }
-        attrs.update(dict((f.name, f.field_instance())
-                            for f in self.fielddefinitions.select_subclasses()))
+        attrs.update(
+            (field_def.name, field_def.construct())
+            for field_def in self.fielddefinitions.select_subclasses()
+        )
         return attrs
 
-    def _create_model_class(self, existing_model_class=None):
+    def construct(self, existing_model_class=None):
         bases = self.get_model_bases()
-        has_mutable_base = False
-        for base in bases:
-            if issubclass(base, MutableModel):
-                has_mutable_base = True
-                base._dependencies.add((self.__class__, self.pk))
-        if not has_mutable_base:
-            bases += (MutableModel,)
-        attrs = self.get_model_attrs(existing_model_class)
+        opts = self.get_model_opts()
+        attrs = self.get_model_attrs()
+
+        identifier = (
+            self.pk, self.object_name, opts, dict(
+                    (name, attr.deconstruct())
+                    for name, attr in attrs.iteritems()
+                        if hasattr(attr, 'deconstruct')
+                ), [
+                MutableModelProxy(base).checksum()
+                    if base is not MutableModel and issubclass(base, MutableModel)
+                    else base
+                for base in bases
+            ]
+        )
+        checksum = md5(pickle.dumps(identifier)).hexdigest()
+        state_handler.set_checksum(self.pk, checksum)
+
         if existing_model_class:
+            if existing_model_class._checksum == checksum:
+                existing_model_class._is_obsolete = False
+                return existing_model_class
+            remove_from_app_cache(existing_model_class)
             existing_model_class.mark_as_obsolete()
+
+        attrs.update(
+            Meta=type(str('Meta'), (), opts),
+            _checksum=checksum
+        )
         model_class = type(str(self.object_name), bases, attrs)
-        mutable_class_prepared.send(sender=model_class, definition=self,
-                                    existing_model_class=existing_model_class)
+
+        mutable_class_prepared.send(
+            sender=model_class, definition=self,
+            existing_model_class=existing_model_class
+        )
         logger.debug("Created model class %s.", model_class)
+
         return model_class
 
     def model_class(self, force_create=False):
         model_class = super(ModelDefinition, self).model_class()
-        if force_create or model_class is None:
-            model_class = self._create_model_class(model_class)
-        return _ModelClassProxy(model_class)
+        if force_create or model_class is None or model_class.is_obsolete():
+            model_class = self.construct(model_class)
+        return MutableModelProxy(model_class)
 
     @property
     def model_ct(self):
@@ -266,8 +327,11 @@ class ModelDefinition(ContentType):
 
     def delete(self, *args, **kwargs):
         model_class = self.model_class()
+        pk = self.pk
         delete = super(ModelDefinition, self).delete(*args, **kwargs)
+        remove_from_app_cache(model_class)
         model_class.mark_as_obsolete()
+        state_handler.clear_checksum(pk)
         ContentType.objects.clear_cache()
         del self._model_class
         return delete
@@ -278,8 +342,10 @@ class ModelDefinitionAttribute(models.Model):
     A mixin used to make sure models that alter the state of a defined model
     clear the cached version
     """
-    model_def = models.ForeignKey(ModelDefinition, related_name="%(class)ss",
-                                  on_delete=CASCADE_MARK_ORIGIN)
+    model_def = models.ForeignKey(
+        ModelDefinition, related_name="%(class)ss",
+        on_delete=CASCADE_MARK_ORIGIN
+    )
 
     class Meta:
         abstract = True
@@ -297,7 +363,16 @@ class ModelDefinitionAttribute(models.Model):
         return delete
 
 
-class BaseDefinition(OrderableModel, ModelDefinitionAttribute):
+class OrderedModelDefinitionAttribute(OrderedModel, ModelDefinitionAttribute):
+    class Meta:
+        abstract = True
+
+    def get_ordering_queryset(self):
+        qs = super(OrderedModelDefinitionAttribute, self).get_ordering_queryset()
+        return qs.filter(model_def_id=self.model_def_id)
+
+
+class BaseDefinition(OrderedModelDefinitionAttribute):
     """
     Model used to represent bases of a ModelDefinition
     """
@@ -305,12 +380,24 @@ class BaseDefinition(OrderableModel, ModelDefinitionAttribute):
 
     class Meta:
         app_label = 'mutant'
-        ordering = ('order',)
+        ordering = ['order']
         unique_together = (('model_def', 'order'),)
 
-    def get_model_class(self):
-        if isinstance(self.base, _ModelClassProxy):
-            return self.base.__get__(None, None)
+    def clean(self):
+        try:
+            if issubclass(self.base, models.Model):
+                if self.base._meta.proxy:
+                    raise ValidationError(_("Base can't be a proxy model."))
+                elif (issubclass(self.base, MutableModel) and
+                      self.base.definition() == self.model_def):
+                    raise ValidationError(_("A model definition can't be a base of itself."))
+        except TypeError:
+            raise ValidationError(_('Base must be a class.'))
+        return super(BaseDefinition, self).clean()
+
+    def construct(self):
+        if isinstance(self.base, MutableModelProxy):
+            return self.base.__get__()
         return self.base
 
     def get_declared_fields(self):
@@ -332,31 +419,20 @@ class BaseDefinition(OrderableModel, ModelDefinitionAttribute):
                 attr_name = '%s_ptr' % model_name(opts)
                 fields.append(
                     models.OneToOneField(
-                        self.base, name=attr_name, null=True,
+                        self.base, name=attr_name,
                         auto_created=True, parent_link=True
                     )
                 )
         return tuple(fields)
 
-    def clean(self):
-        try:
-            if issubclass(self.base, models.Model):
-                if self.base._meta.proxy:
-                    raise ValidationError(_("Base can't be a proxy model."))
-                elif (issubclass(self.base, MutableModel) and
-                      self.base.definition() == self.model_def):
-                    raise ValidationError(_("A model definition can't be a base of itself."))
-        except TypeError:
-            raise ValidationError(_('Base must be a class.'))
-        return super(BaseDefinition, self).clean()
 
-
-class OrderingFieldDefinition(OrderableModel, ModelDefinitionAttribute):
+class OrderingFieldDefinition(OrderedModelDefinitionAttribute):
     lookup = models.CharField(max_length=255)
     descending = models.BooleanField(_('descending'), default=False)
 
-    class Meta(OrderableModel.Meta):
+    class Meta:
         app_label = 'mutant'
+        ordering = ['order']
         # TODO: Should be unique both it bugs order swapping
         # unique_together = (('model_def', 'order'),)
 
@@ -366,7 +442,6 @@ class OrderingFieldDefinition(OrderableModel, ModelDefinitionAttribute):
         """
         if self.lookup == '?':  # Randomly sort
             return
-        #TODO: Support order_with_respect_to...
         else:
             lookups = self.lookup.split(LOOKUP_SEP)
             opts = self.model_def.model_class()._meta
@@ -384,10 +459,10 @@ class OrderingFieldDefinition(OrderableModel, ModelDefinitionAttribute):
                         valid = False
                 finally:
                     if not valid:
-                        msg = _('This field doesn\'t exist')
+                        msg = _("This field doesn't exist")
                         raise ValidationError({'lookup': [msg]})
 
-    def get_defined_ordering(self):
+    def construct(self):
         return ("-%s" % self.lookup) if self.descending else self.lookup
 
 
@@ -401,7 +476,7 @@ class UniqueTogetherDefinition(ModelDefinitionAttribute):
 
     def __unicode__(self):
         if self.pk:
-            names = ', '.join(self.field_defs.names())
+            names = ', '.join(self.construct())
             return _("Unique together of (%s)") % names
         return ''
 
@@ -410,3 +485,6 @@ class UniqueTogetherDefinition(ModelDefinitionAttribute):
             if field_def.model_def != self.model_def:
                 msg = _('All fields must be of the same model')
                 raise ValidationError({'field_defs': [msg]})
+
+    def construct(self):
+        return self.field_defs.names()
