@@ -54,11 +54,11 @@ class FieldDefinitionTestMixin(object):
 
     def setUp(self):
         super(FieldDefinitionTestMixin, self).setUp()
-        self.field = self.field_definition_cls._default_manager.create(
-            model_def=self.model_def,
-            name='field',
-            **self.field_definition_init_kwargs
-        )
+        with self.assertChecksumChange():
+            self.field = self.field_definition_cls._default_manager.create(
+                model_def=self.model_def, name='field',
+                **self.field_definition_init_kwargs
+            )
 
     def get_field_value(self, instance, name='field'):
         return getattr(instance, name)
@@ -70,103 +70,120 @@ class FieldDefinitionTestMixin(object):
         default = self.prepare_default_value(self.field_values[0])
         field = self.field
         # Default value should be valid
-        field.default = default
-        field.full_clean()
-        field.save()
+        with self.assertChecksumChange():
+            field.default = default
+            field.full_clean()
+            field.save()
         # Default value should be assigned correctly
-        Model = self.model_def.model_class()
-        instance = Model.objects.create()
-        created_default = self.prepare_default_value(self.get_field_value(instance))
+        model_class = self.model_def.model_class()
+        instance = model_class.objects.create()
+        field_value = self.get_field_value(instance)
+        created_default = self.prepare_default_value(field_value)
         self.assertEqual(created_default, default)
 
     def test_create_with_default(self):
-        """
-        Makes sure a field definition manager is attached to the model and
-        `create_with_default` works correctly.
-        """
-        Model = self.model_def.model_class()
+        """Makes sure a field definition manager is attached to the model and
+        `create_with_default` works correctly."""
+        model_class = self.model_def.model_class()
         field_value = self.field_values[0]
-        instance = Model.objects.create(field=field_value)
+        instance = model_class.objects.create(field=field_value)
         # Add the field with a default.
         create_default = self.prepare_default_value(field_value)
         options = dict(**self.field_definition_init_kwargs)
         options['default'] = create_default
-        self.field_definition_cls._default_manager.create_with_default(
-            model_def=self.model_def, name='field_created_with_default',
-            **options
-        )
+        with self.assertChecksumChange():
+            self.field_definition_cls._default_manager.create_with_default(
+                model_def=self.model_def, name='field_created_with_default',
+                **options
+            )
         created_value = self.prepare_default_value(
-            Model.objects.get(pk=instance.pk).field_created_with_default
+            model_class.objects.get(pk=instance.pk).field_created_with_default
         )
         self.assertEqual(created_value, create_default)
 
     def test_model_save(self):
         first_value, second_value = self.field_values
         # Assigning a value should work
-        Model = self.model_def.model_class()
-        instance = Model.objects.create(field=first_value)
+        model_class = self.model_def.model_class()
+        instance = model_class.objects.create(field=first_value)
         self.assertEqual(self.get_field_value(instance), first_value)
         # Assigning a new one should also work
         instance.field = second_value
         instance.save()
-        instance = Model.objects.get()
+        instance = model_class.objects.get()
         self.assertEqual(self.get_field_value(instance), second_value)
 
     def test_field_renaming(self):
         value = self.field_values[0]
-        Model = self.model_def.model_class()
+        model_class = self.model_def.model_class()
         # Renaming a field should update its column name
-        Model.objects.create(field=value)
-        _, original_column_name = Model._meta.get_field('field').get_attname_column()
-        self.field.name = 'renamed_field'
-        self.field.save()
-        _, new_column_name = Model._meta.get_field('renamed_field').get_attname_column()
-        self.assertModelTablesColumnDoesntExists(Model, original_column_name)
-        self.assertModelTablesColumnExists(Model, new_column_name)
+        model_class.objects.create(field=value)
+        opts = model_class._meta
+        original_column_name = opts.get_field('field').get_attname_column()[1]
+        with self.assertChecksumChange():
+            self.field.name = 'renamed_field'
+            self.field.save()
+        opts = model_class._meta
+        new_column_name = opts.get_field('renamed_field').get_attname_column()[1]
+        self.assertModelTablesColumnDoesntExists(
+            model_class, original_column_name
+        )
+        self.assertModelTablesColumnExists(
+            model_class, new_column_name
+        )
         # Old data should be accessible by the new field name
-        instance = Model.objects.get()
+        instance = model_class.objects.get()
         self.assertEqual(self.get_field_value(instance, 'renamed_field'), value)
         # The old field shouldn't be accessible anymore
         msg = "'field' is an invalid keyword argument for this function"
-        self.assertRaisesMessage(TypeError, msg, Model, field=value)
+        self.assertRaisesMessage(TypeError, msg, model_class, field=value)
         # It should be possible to create objects using the new field name
-        Model.objects.create(renamed_field=value)
+        model_class.objects.create(renamed_field=value)
 
     def test_field_deletion(self):
         value = self.field_values[0]
-        Model = self.model_def.model_class()
-        Model.objects.create(field=value)
+        model_class = self.model_def.model_class()
+        model_class.objects.create(field=value)
         # Deleting a field should delete the associated column
-        _, field_column_name = Model._meta.get_field('field').get_attname_column()
-        self.field.delete()
-        self.assertModelTablesColumnDoesntExists(Model, field_column_name)
+        opts = model_class._meta
+        field_column_name = opts.get_field('field').get_attname_column()[1]
+        with self.assertChecksumChange():
+            self.field.delete()
+        self.assertModelTablesColumnDoesntExists(model_class, field_column_name)
         # The deleted field shouldn't be accessible anymore
         msg = "'field' is an invalid keyword argument for this function"
-        self.assertRaisesMessage(TypeError, msg, Model, field=value)
+        self.assertRaisesMessage(TypeError, msg, model_class, field=value)
 
     def test_field_unique(self):
         value = self.field_values[0]
-        Model = self.model_def.model_class()
-        self.field.unique = True
-        self.field.save()
-        Model.objects.create(field=value)
-        write_db = router.db_for_write(Model)
+        model_class = self.model_def.model_class()
+        with self.assertChecksumChange():
+            self.field.unique = True
+            self.field.save()
+        model_class.objects.create(field=value)
+        write_db = router.db_for_write(model_class)
+        connection = connections[write_db]
+        # TODO: Convert this to `atomic` once support for Django < 1.6 is dropped
         sid = transaction.savepoint(using=write_db)
         try:
-            Model.objects.create(field=value)
+            model_class.objects.create(field=value)
         except IntegrityError:
             pass
         else:
             self.fail("One shouldn't be able to save duplicate entries in a unique field")
         finally:
+            connection.needs_rollback = False
             transaction.savepoint_rollback(sid, using=write_db)
 
     def test_field_cloning(self):
-        clone = self.field.clone()
-        clone.name = 'field_clone'
-        clone.model_def = self.model_def
-        clone.save(force_insert=True)
+        with self.assertChecksumChange():
+            clone = self.field.clone()
+            clone.name = 'field_clone'
+            clone.model_def = self.model_def
+            clone.save(force_insert=True)
 
     def test_field_definition_category(self):
-        self.assertEqual(self.field_definition_cls.get_field_category(),
-                         self.field_definition_category)
+        self.assertEqual(
+            self.field_definition_cls.get_field_category(),
+            self.field_definition_category
+        )
