@@ -1,5 +1,7 @@
 from __future__ import unicode_literals
 
+import operator
+
 from contextlib import contextmanager
 from copy import deepcopy
 from itertools import groupby
@@ -7,10 +9,24 @@ import imp
 from operator import itemgetter
 
 import django
+from django.db import connections, router
 from django.db.models.loading import cache as app_cache
 from django.utils.datastructures import SortedDict
 from django.utils.encoding import force_unicode
-from django.utils.functional import lazy
+from django.utils.functional import lazy, LazyObject, new_method_proxy
+
+
+# TODO: Remove `allow_syncdb` alternative when support for 1.6 is dropped
+if django.VERSION >= (1, 7):
+    def allow_migrate(model):
+        for db in connections:
+            if router.allow_migrate(db, model):
+                yield db
+else:
+    def allow_migrate(model):
+        for db in connections:
+            if router.allow_syncdb(db, model):
+                yield db
 
 
 NOT_PROVIDED = object()
@@ -120,3 +136,32 @@ if django.VERSION >= (1, 6):  # pragma: no cover
 else:  # pragma: no cover
     def model_name(opts):
         return opts.module_name
+
+
+_opts_related_cache_attrs = [
+    '_related_objects_cache', '_related_objects_proxy_cache',
+    '_related_many_to_many_cache', '_name_map'
+]
+
+
+def clear_opts_related_cache(model_class):
+    """
+    Clear the specified model opts related cache
+    """
+    opts = model_class._meta
+    for attr in _opts_related_cache_attrs:
+        try:
+            delattr(opts, attr)
+        except AttributeError:
+            pass
+
+# TODO: Remove when support for 1.5 is dropped
+if django.VERSION < (1, 6):
+    class LazyObject(LazyObject):
+        # Dictionary methods support
+        __getitem__ = new_method_proxy(operator.getitem)
+        __setitem__ = new_method_proxy(operator.setitem)
+        __delitem__ = new_method_proxy(operator.delitem)
+
+        __len__ = new_method_proxy(len)
+        __contains__ = new_method_proxy(operator.contains)
